#!/usr/bin/perl
#
###############################################################################
#
# epccontrol3.pl v0.1                                          
#
# CommandLine Tool to access the ExpertPowerControl series (Rev3)
#
# sample calls:
#    Show current State:
#       - epccontrol3.pl --host=192.168.0.2 --password=xyz
#
#    Switch on Powerport1 :
#       - epccontrol3.pl --host=192.168.0.2 --password=xyz -p=1 -s=1
#
#    Switch off Powerport1 :
#       - epccontrol3.pl --host=192.168.0.2 --password=xyz -p=1 -s=0
#
###############################################################################
#
# (c) 2004,2008,2012 Author: Martin Bachem
#    Gude Analog- und Digitalsysteme GmbH
#    http://www.gude.info/
#    info@gudeads.com
#
#  epccontrol3 is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation version 2
#
#  epccontrol2 is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with epccontrolv2.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

eval("use LWP::UserAgent");
if ($@) {
  print "ERROR: Module LWP::UserAgent not on found this host\n";
  exit;
}
eval("use JSON");
if ($@) {
  print "ERROR: Module JSON not on found this host\n";
  exit;
}


$password = "";
$powerport=0;
$powerstate=-1;
$action=0;
$batch_delay=1;


sub usage {
  print ("epccontrol3.pl\n");
  print ("  [--host=192.168.0.2]      : define the IP-Adress of your device)\n");
  print ("  [--password=mypasswd]     : define HTTP Password if needed\n");
  print ("  [--port=X],[-p=X]         : select powerport no (default: 0, just shows states)\n");
  print ("  [--switch=S], [-s=S]      : switch to state 'S' (0=off, 1=on)\n");
  print ("  [--batch_onoff=S],[-bn=S] : switch port on, after S seconds switch off\n");
  print ("  [--batch_offon=S],[-bf=S] : switch port off, after S seconds switch on\n");
  print ("\n");
  exit;
}

# parse cmd line params
foreach $cmd_line_param (@ARGV) {
  # --host=192.168.0.2
  if ($cmd_line_param =~ "--host=") {
    @param_vals = split(/=/, $cmd_line_param);
    if ($param_vals[1] ne "") {
      $host = $param_vals[1];
    }
  }

  # --password=mypasswd
  if ($cmd_line_param =~ "--password=") {
    @param_vals = split(/=/, $cmd_line_param);
    if ($param_vals[1] ne "") {
      $password = $param_vals[1];
    }
  }

  # [--port=X], [-p=X]
  if (($cmd_line_param =~ "--port")  || ($cmd_line_param =~ "-p=")) {
    @param_vals = split(/=/, $cmd_line_param);
    if ($param_vals[1] ne "") {
      $powerport = $param_vals[1];
    }
  }

  # [--switch], [-s]
  if (($cmd_line_param =~ "--switch") || ($cmd_line_param =~ "-s")) {
    $action = 1;
    @param_vals = split(/=/, $cmd_line_param);
    $powerstate = $param_vals[1];
  }

  # [--switch_on], [-on]  --> action = 1;
  if (($cmd_line_param eq "--switch_on") || ($cmd_line_param eq "-on")) {
    $action = 1;
    $powerstate = 1;
  }

  # [--switch_off], [-off]  --> action = 2;
  if (($cmd_line_param eq "--switch_off") || ($cmd_line_param eq "-off")) {
    $action = 1;
    $powerstate = 0;
  }

  # [--batch_onoff=T], [-bon=X] : reset port (switch on, wait, switch off)
  if (($cmd_line_param =~ "--batch_onoff=")  || ($cmd_line_param =~ "-bn=")) {
    @param_vals = split(/=/, $cmd_line_param);
    if ($param_vals[1] ne "") {
      $action = 2;
      $batch_delay = $param_vals[1];
    }
  }

  # [--batch_onoff=T], [-bon=X] : reset port (switch on, wait, switch off)
  if (($cmd_line_param =~ "--batch_offon=")  || ($cmd_line_param =~ "-bf=")) {
    @param_vals = split(/=/, $cmd_line_param);
    if ($param_vals[1] ne "") {
      $action = 3;
      $batch_delay = $param_vals[1];
    }
  }

  # [--help], [-h]  --> print usage;
  if (($cmd_line_param eq "--help") || ($cmd_line_param eq "-h")) {
    usage();
  }
}

if ($host eq "") {
  usage();
  exit;
}

if (!($host =~ ":"))  {
  $host = $host.':80';
}

if (($action == 1) && ($powerstate != 1) && ($powerstate != 0)) {
  usage();
}

################################################################################################

$browser = LWP::UserAgent->new;
$browser->credentials(
  $host,
  'Enter Password',
  'user' => $password
);
$browser->agent($ENV{SCRIPT_NAME});

# fallback url: just display powerport-states
$get_url = "http://".$host."/statusjsn.js?components=1";

if ($action && !$powerport) {
  $powerport = 1;
}

if ($action == 1) {
  # simple switch
  $get_url .= "&cmd=1&p=".$powerport."&s=".$powerstate;
} elsif ($action == 2) {
  # batchmode on->wait->off
  $get_url .= "&cmd=5&p=".$powerport."&s=".$batch_delay."&a1=1&a2=0";
} elsif ($action == 3) {
  # batchmode off->wait->on
  $get_url .= "&cmd=5&p=".$powerport."&s=".$batch_delay."&a1=0&a2=1";
}

print $get_url . "\n";
$response = $browser->get($get_url);

if ($response->is_success) {
  # print $response->content;
  $json = decode_json($response->content);
  foreach my $output(@{$json->{outputs}}) {
    print $output->{name} . " is " . ($output->{state} ? "on" : "off");
    if ($output->{batch}[0]) {
      my $nextState = "off";
      if ($output->{batch}[2] == 2) {
        my $nextState = "on";
      }
      print "  -  will switch " . $nextState . " in " . $output->{batch}[1] . "s";
    }
    print "\n";
  }
} else {
  print "Error with HTTP-Return-Code: " . $response->code . "\n";
  if ($response->code == 401) {
    print "please provide password for 'user' with e.g. --password=verysecret\n";
    print "\n";
  }
  # print $response->content."\n";
}

